@@ -12,7 +12,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: bottleparser.py,v 1.14 2004/08/17 12:43:48 atzm Exp $
+# $Id: bottleparser.py,v 1.15 2004/09/01 02:35:10 atzm Exp $
 #
 
 from script import *
@@ -118,11 +118,11 @@ class BottleParser(Parser):
 					script.append((PARSE_ERROR, lexeme, argument))
 				else:
 					script.append((SCRIPT_TAG, lexeme, argument))
-			elif lexeme[:2] in ["\\w"]: #,"\\s"]:
+			elif lexeme[:2] in ["\\w", "\\s"]:
 				#print token, lexeme
 				num = lexeme[2]
-				#if lexeme[:2] == "\\s" and scope == 1:
-				#	num = str(int(num) + 10)
+				if lexeme[:2] == "\\s" and scope == 1:
+					num = str(int(num) + 10)
 				script.append((SCRIPT_TAG, lexeme[:2], num))
 			elif lexeme in ["\\n"]:
 				if self.tokens and self.tokens[0][0] == TOKEN_OPENED_SBRA:
