@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: common.py,v 1.25 2004/10/31 13:28:17 atzm Exp $
+# $Id: common.py,v 1.26 2004/11/29 07:36:02 atzm Exp $
 #
 
 import os, sys
@@ -19,7 +19,7 @@ import gtk
 
 # common defines
 APP = 'GBottler'
-VER = '0.1.3.1'
+VER = '0.1.4'
 
 VOTE  = "Vote"
 AGREE = "Agree"
