@@ -13,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: gbottler.py,v 1.158 2004/09/27 13:11:38 atzm Exp $
+# $Id: gbottler.py,v 1.165 2004/10/28 12:04:09 atzm Exp $
 #
 
 import os, sys, string, re, gettext, getopt
@@ -31,13 +31,12 @@ from logwindow      import LogWindow
 from phrases        import PhraseWindow
 from prefs          import PreferenceManager, PreferenceDialog
 from textmanager    import TextManager
-from messagesender  import MessageSender, VoteMessageSender
-from queryluid      import LUIDInfo
-from gvwindow       import GVWindow
-from svgmanager     import SVGManager
-from bottleparser   import BottleParser
 from scriptchecker  import ScriptChecker
 
+from viewer.ghostmanager import GhostManager
+from gvwindow            import GVWindow
+from svgmanager          import SVGManagerWindow
+
 from common import *
 
 if os.name == 'posix':
@@ -54,7 +53,6 @@ class Application:
 # === Definition of Basic Const === #
 		self.prefs         = PreferenceManager()
 		self.scriptchecker = ScriptChecker()
-		self.color_parser  = BottleParser('loose')  # using coloring
 
 		self.accel_group = gtk.AccelGroup()
 
@@ -182,42 +180,44 @@ class Application:
 		self.textmanager.insert_with_color_from(str, pos)
 		self.editor_modified()
 
-		self.client = Bottler(self, self.prefs)
+		self.client = Bottler(self, self.prefs.get(PreferenceManager.PREF_DOWNLOAD),
+							  self.prefs.get_all(),
+							  self.prefs.get(PreferenceManager.PREF_LOGGING_DOWNLOAD))
 		self.client.debug = 2
 
-		self.luid_info = None
 		try:
 			luid = open(os.path.join(open_bottlecase(), "luid")).read()
 		except IOError:
 			luid = None
 		if luid and len(luid) == 82:
 			self.client.luid = luid
-			self.luid_info = LUIDInfo(luid)
 
-		self.log_window = LogWindow(self)
-		self.gvwindow = GVWindow(os.path.join(open_bottlecase(), 'gviewer', 'Ghost'),
-								 os.path.join(open_bottlecase(), 'gviewer', 'ghost.txt'))
+		self.logwindow    = LogWindow(self, self.prefs)
 		self.phrasewindow = PhraseWindow(self)
-		self.svgmanager = SVGManager(self, self.prefs, os.path.join(open_bottlecase(), 'gviewer'))
+
+		ghostmanager = GhostManager(os.path.join(open_bottlecase(), 'gviewer', 'ghost.txt'),
+									os.path.join(open_bottlecase(), 'gviewer', 'Ghost'))
+		self.svgmanager = SVGManagerWindow(self, self.prefs, ghostmanager)
+		self.gvwindow   = GVWindow(ghostmanager) # must watch same instance of GhostManager
 
 		self.menu_list["/%s/%s..." % (self.msg["window"][1],
-									  self.msg["logs"])].connect("activate", self.log_window.open, None)
+									  self.msg["logs"])].connect("activate", self.logwindow.open, None)
 
 		self.menu_list["/%s/%s..." % (self.msg["window"][1],
 									  self.msg["phrases"])].connect("activate", self.phrasewindow.open, None)
 
 		self.menu_list["/%s/%s..." % (self.msg["window"][1],
-									  self.msg["gviewer"])].connect("activate", self.gvwindow.open, None)
+									  self.msg["gviewer"])].connect("activate", self.open_gviewer, None)
 
 		self.menu_list["/%s/%s..." % (self.msg["window"][1],
 									  self.msg["svgmanager"])].connect("activate", self.svgmanager.open, None)
 
-		self.localmessenger = LocalMessenger(host, port, self.client, self.log_window)
+		self.localmessenger = LocalMessenger(host, port, self.logwindow)
 
 		self.target_ghost = self.DEFAULT_TARGET
 
 		self.local_ghost = []
-		self.fmo_ghosts = self.gvwindow.get_fmo_ghosts()
+		self.fmo_ghosts = ghostmanager.get_fmo_ghostnames()
 
 		self.channel_event_id = 1 # channel label eventbox handler id
 		self.ghost_event_id = 1
@@ -230,10 +230,10 @@ class Application:
 		self.localghost_timeout_id = gtk.timeout_add(WAIT_NEXT_LOCALGHOST_POLL, self.handle_localghost)
 
 # === Initialization from Preferences === #
-		if self.prefs.get('auto_open_log'):
-			self.log_window.open(None, None)
+		if self.prefs.get(PreferenceManager.PREF_STARTUP_OPEN_LOG):
+			self.logwindow.open(None, None)
 
-		if self.prefs.get('auto_connect'):
+		if self.prefs.get(PreferenceManager.PREF_STARTUP_AUTO_JOIN):
 			self.join(None, None)
 
 		self.notify_preferences_changed(self.prefs.get_all())
@@ -254,13 +254,20 @@ class Application:
 
 	def quit(self, data, widget):
 		self.part(data, widget)
-		gtk.mainquit()
+		gtk.main_quit()
+
+	def open_gviewer(self, widget=None, data=None):
+		self.gvwindow.open()
+		if self.prefs.get(PreferenceManager.PREF_GVIEWER_STICKY):
+			self.gvwindow.stick()
+		else:
+			self.gvwindow.unstick()
 
 	def open_preference(self, widget=None, data=None):
-		PreferenceDialog(self, self.prefs, self.luid_info)
+		PreferenceDialog(self, self.prefs, self.client.luid)
 
 	def get_initial_script(self):
-		scr = self.prefs.get('initialscript')
+		scr = self.prefs.get(PreferenceManager.PREF_INITIALSCRIPT)
 		n = string.find(scr, '|')
 
 		# | found
@@ -276,35 +283,28 @@ class Application:
 		return [scr, n]
 
 	def notify_log_current_tab_closed(self):
-		if self.prefs.get('logging_current_tab_closed'):
-			self.log_window.autologging(self.prefs.get('logging_path'), self.prefs.get('logging_compress'))
+		if self.prefs.get(PreferenceManager.PREF_LOGGING_CURRENT_CLOSED):
+			self.logwindow.autologging(self.prefs.get(PreferenceManager.PREF_LOGGING_PATH),
+									   self.prefs.get(PreferenceManager.PREF_LOGGING_COMPRESS))
 
 	def notify_preferences_changed(self, prefs):
 		# notify BottleClient
-		self.client.set_pattern(prefs)
-		self.client.set_dl_logging(prefs['download_log'])
+		self.client.receive_preferences_changed(prefs)
 
 		# notify LogWindow
-		self.log_window.tab_pos_changed(int(prefs['logtabpos']))
-		self.log_window.color_changed(prefs['script'], prefs['sakura'], prefs['kero'],
-									  prefs['sync'], prefs['error'], prefs['URL'])
-		self.log_window.font_changed(prefs['general font'], prefs['script font'])
+		self.logwindow.receive_preferences_changed(prefs)
 
 		# notify GVWindow
-		self.gvwindow.notify_use_gviewer(prefs['use_gviewer'])
-		self.gvwindow.notify_wait_1_char(prefs['wait_1_char'])
-		self.gvwindow.notify_wait_w1_tag(prefs['wait_w1_tag'])
-		self.gvwindow.notify_font_changed(prefs['general font'])
+		self.gvwindow.receive_preferences_changed(prefs)
 
-		# notify self attributes
-		self.textmanager.color_changed(prefs['script'], prefs['sakura'], prefs['kero'],
-									   prefs['sync'], prefs['error'], prefs['URL'])
-		self.textmanager.font_changed(prefs['general font'], prefs['script font'])
-		self.textmanager.coloring()
+		# notify SVGManagerWindow
+		self.svgmanager.browser_changed(prefs[PreferenceManager.PREF_BROWSER_COMMAND])
+
+		# notify TextManager
+		self.textmanager.receive_preferences_changed(prefs)
 
 	def notify_svg_changed(self):
-		self.gvwindow.notify_svg_changed()
-		self.fmo_ghosts = self.gvwindow.get_fmo_ghosts()
+		self.fmo_ghosts = self.svgmanager.get_fmo_ghostnames()
 		self.rebuild_ghostmenu()
 
 	def notify_play_cancel(self, mid):
@@ -318,7 +318,7 @@ class Application:
 		self.gvwindow.notify_cancel_all()
 
 	def notify_log_update(self, mid, channel, ifghost, script, receive_time):
-		self.log_window.update(mid, channel, ifghost, script, receive_time)
+		self.logwindow.update(mid, channel, ifghost, script, receive_time)
 
 	def dispose_selection(self):
 		for target in ["CLIPBOARD", "PRIMARY"]:
@@ -376,7 +376,6 @@ class Application:
 		if not self.client.initialized:
 			try:
 				self.client.init()
-				self.luid_info = LUIDInfo(self.client.luid)
 			except BottleClientError, e:
 				return
 			filename = os.path.join(open_bottlecase(), "luid")
@@ -386,7 +385,7 @@ class Application:
 				self.monitor_insert("\n" + unicode(_("Warning"), "utf-8") + \
 									": cannot write " + filename)
 
-		if self.prefs.get('auto_join'):
+		if self.prefs.get(PreferenceManager.PREF_AUTO_JOIN_ALL):
 			channels = range(len(self.client.channels))
 		else:
 			d = ChannelDialog(self.window, self.client.channels)
@@ -418,8 +417,9 @@ class Application:
 		self.monitor_insert(unicode(_("Connection closed."), "utf8"))
 		self.disable_channel()
 
-		if self.prefs.get('logging'):
-			self.log_window.autologging(self.prefs.get('logging_path'), self.prefs.get('logging_compress'))
+		if self.prefs.get(PreferenceManager.PREF_LOGGING_DISCONNECTED):
+			self.logwindow.autologging(self.prefs.get(PreferenceManager.PREF_LOGGING_PATH),
+									   self.prefs.get(PreferenceManager.PREF_LOGGING_COMPRESS))
 
 	def enable_channel(self):
 		channel_item = self.menu_list["/%s" % self.msg["channel"][1]]
@@ -460,13 +460,13 @@ class Application:
 		self.rebuild_ghostmenu()
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["part"])].set_sensitive(gtk.TRUE)
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["join"])].set_sensitive(gtk.FALSE)
-		self.log_window.join()
+		self.logwindow.join()
 		self.show_users()
 
 	def disable_channel(self):
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["part"])].set_sensitive(gtk.FALSE)
 		self.menu_list["/%s/%s" % (self.msg["file"][1], self.msg["join"])].set_sensitive(gtk.TRUE)
-		self.log_window.part()
+		self.logwindow.part()
 
 		if self.channel_event_box.handler_is_connected(self.channel_event_id):
 			self.channel_event_box.disconnect(self.channel_event_id)
@@ -541,11 +541,9 @@ class Application:
 					ghosts.append(name)
 
 		ghost_menu = []
-		for n in range(len(ghosts)):
-			if n == 0:
-				ghost_menu.append(("/"+ghosts[n], None, None, 0, "<RadioItem>"))
-			else:
-				ghost_menu.append(("/"+ghosts[n], None, None, 0, "/"+ghosts[n-1]))
+		ghost_menu.append(("/"+ghosts[0], None, None, 0, "<RadioItem>"))
+		for n in range(1, len(ghosts)):
+			ghost_menu.append(("/"+ghosts[n], None, None, 0, "/"+ghosts[n-1]))
 		ghost_menu = tuple(ghost_menu)
 
 		self.g_ifact = gtk.ItemFactory(gtk.Menu, "<main>")
@@ -604,7 +602,7 @@ class Application:
 
 		title = regexp.sub('', open_simple_entry_dialog(unicode(_('Please input title'), 'utf-8'),
 														unicode(_('Please input title'), 'utf-8'),
-														self.window))
+														parent=self.window))
 		script = regexp.sub('', self.textmanager.get_script())
 
 		if title and script:
@@ -634,30 +632,35 @@ class Application:
 		ghost = self.target_ghost
 		if ghost == self.DEFAULT_TARGET:
 			ghost = None
-		ms = MessageSender(self, widget, data, channel, script, ghost)
-		ms.start()
+
+		self.client.send_message(self.prefs.get(PreferenceManager.PREF_AUTO_EDITOR_CLEAR),
+								 self.prefs.get(PreferenceManager.PREF_LOGGING_SENT),
+								 channel, script, ghost)
 
 	def send_local_message(self, channel, ifghost, script, sender=APP, charset='Shift_JIS', mid=None, list=None):
 		if list is None:
-			list = self.log_window.logbook.get_current_loglist()
+			list = self.logwindow.logbook.get_current_loglist()
 
-		try:
-			self.localmessenger.send_local_message(channel, ifghost, script, sender, charset, mid, list)
-		except BottleClientError, e:
- 			open_error_dialog(str(e), self.window)
- 			return
+		if self.prefs.get(PreferenceManager.PREF_FORWARD):
+			if self.prefs.get(PreferenceManager.PREF_FORWARD_ACCEPT_ALL) or \
+				   (self.prefs.get(PreferenceManager.PREF_FORWARD_ONLY_LISTED) and \
+					ifghost in self.local_ghost):
+				try:
+					self.localmessenger.send_local_message(channel, ifghost, script, sender, charset, mid, list)
+				except BottleClientError, e:
+					open_error_dialog(str(e), self.window)
+					return
 
-		if self.prefs.get('use_gviewer'):
-			if self.prefs.get('open_gviewer_when_play'):
-				self.gvwindow.open()
+		if self.prefs.get(PreferenceManager.PREF_GVIEWER_USE):
+			if self.prefs.get(PreferenceManager.PREF_GVIEWER_OPEN_PLAY):
+				self.open_gviewer()
 			self.gvwindow.enqueue(channel, ifghost, script, sender)
 
 	def log_votes(self, mid, type, num):
-		self.log_window.log_votes(mid, type, num)
+		self.logwindow.log_votes(mid, type, num)
 
 	def vote_message(self, mid, type):
-		vms = VoteMessageSender(self.client, self.log_window, mid, type)
-		vms.start()
+		self.client.send_vote_message(self.logwindow, mid, type)
 
 	def test(self, widget=None, data=None):
 		self.textmanager.grab_focus()
@@ -1002,7 +1005,7 @@ class Application:
 			( "/%s/%s" % (self.msg["edit"][1], self.msg["cut"]), "<control>X", self.edit_cut, 0, "<StockItem>", gtk.STOCK_CUT ),
 			( "/%s/%s" % (self.msg["edit"][1], self.msg["copy"]), "<control>C", self.edit_copy, 0, "<StockItem>", gtk.STOCK_COPY ),
 			( "/%s/%s" % (self.msg["edit"][1], self.msg["paste"]), "<control>V", self.edit_paste, 0, "<StockItem>", gtk.STOCK_PASTE ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["delete"]), "<control>D", self.edit_delete,0, "<StockItem>", gtk.STOCK_DELETE ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["delete"]), "Delete", self.edit_delete,0, "<StockItem>", gtk.STOCK_DELETE ),
 			( "/%s/sep4" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
 #			( "/%s/collect" % (self.msg["edit"][1]), None, lambda x, y: gc.collect(), 0, None ),
 			( "/%s/%s..." % (self.msg["edit"][1], self.msg["pref"][0]), None, self.open_preference, 0, "<StockItem>", gtk.STOCK_PREFERENCES ),
