@@ -11,48 +11,39 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: svgmanager.py,v 1.32 2004/09/21 07:29:43 tkawa Exp $
+# $Id: svgmanager.py,v 1.43 2004/10/28 20:52:54 atzm Exp $
 
-import pygtk
-import gtk
-import os
-import shutil
-import sys
-import string
-import re
+import gtk, gobject
+import os, sys, shutil
+import string, re
 
-from common              import *
 from showingsurface      import ShowingSurface
-from viewer.ghostmanager import GhostManager
+from common              import *
 from viewer.viewercommon import *
 
-class SVGManager(gtk.Window):
-	def __init__(self, app, prefs, base_dir, ghost_file='ghost.txt'):
+class SVGManagerWindow(gtk.Window):
+	def __init__(self, app, browse_command, ghostmanager):
 		self.app = app
-		self.prefs = prefs
-		self.base_dir = base_dir
-		self.ghost_file = ghost_file
-		self.gm = GhostManager(self.ghost_file, True)
+		self.browse_command = browse_command
+		self.ghostmanager   = ghostmanager
+
 		accel_group = gtk.AccelGroup()
 
 ## Create status (left) window
 		## Create GhostList window
-		self.gn_str = gtk.ListStore('gboolean', str, str, str)
 
-		self.gn = gtk.TreeView(self.gn_str)
-		self.gn.set_enable_search(gtk.TRUE)
-		self.gn.set_rules_hint(gtk.TRUE)
-		self.gn.set_search_column(gtk.TRUE)
-		self.gn.set_reorderable(gtk.TRUE)
-		self.gn.connect("cursor-changed", self.cc)
-		self.gn.connect("select-cursor-row", self.select_row)
+		self.ghost_list_str = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING,
+									gobject.TYPE_STRING,  gobject.TYPE_STRING)
 
-		# generate ghost_list
-		if not os.path.exists(self.base_dir):
-			os.makedirs(self.base_dir)
-			self.set_config(self.ghost_file)
+		self.ghost_list = gtk.TreeView(self.ghost_list_str)
+		self.ghost_list.set_enable_search(gtk.TRUE)
+		self.ghost_list.set_rules_hint(gtk.TRUE)
+		self.ghost_list.set_search_column(gtk.TRUE)
+		self.ghost_list.set_reorderable(gtk.TRUE)
+		self.ghost_list.connect("cursor-changed", self.cursor_changed)
+		self.ghost_list.connect("select-cursor-row", self.select_row)
 
-		self.get_config(ghost_file)
+		self.get_config()
 
 		# make ghostlist column
 		columns = [
@@ -65,7 +56,7 @@ class SVGManager(gtk.Window):
 			columns[i].set_resizable(gtk.TRUE)
 			columns[i].set_clickable(gtk.TRUE)
 			columns[i].connect("clicked", self.sort, i)
-			self.gn.append_column(columns[i])
+			self.ghost_list.append_column(columns[i])
 
 			if i == 0:
 				cell = gtk.CellRendererToggle()
@@ -78,12 +69,12 @@ class SVGManager(gtk.Window):
 				columns[i].add_attribute(cell, 'text', i)
 				if   i == 1: cell.set_property('width', 80)
 				elif i == 2: cell.set_property('width', 40)
-				else:        cell.set_property('width', 140) # XXX: kimeuchi ikunai...
+				#else:        cell.set_property('width', 140) # FIXME : kimeuchi ikunai
 
 		self.sw = gtk.ScrolledWindow()
 		self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		self.sw.set_shadow_type(gtk.SHADOW_IN)
-		self.sw.add(self.gn)
+		self.sw.add(self.ghost_list)
 
 		# Create GhostInformation main
 		self.sakura_box = gtk.Image()
@@ -130,11 +121,21 @@ class SVGManager(gtk.Window):
 		info_ghost_a.pack_start(info_ghost, gtk.TRUE, gtk.FALSE, 0)
 		info_ghost_a.pack_start(buttonbox, gtk.FALSE, gtk.FALSE, 0)
 
-		#info_svg is 'showing information of selected svg_file (from *.txt)'
-		info_svg = gtk.TextView()
-		info_svg.set_editable(gtk.FALSE)
-		info_svg.set_cursor_visible(gtk.FALSE)
-		self.info_buffer = info_svg.get_buffer()
+		## Showing Ghost Status from svg.txt
+		self.info_svg_str = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+		info_svg = gtk.TreeView(self.info_svg_str)
+		info_svg.set_headers_visible(gtk.FALSE)
+		info_svg.set_rules_hint(gtk.TRUE)
+
+		svg_cols = [
+			gtk.TreeViewColumn(),
+			gtk.TreeViewColumn(unicode(_('Ghost Status'), 'utf-8'))
+			]
+		for i in range(len(svg_cols)):
+			info_svg.append_column(svg_cols[i])
+			cell = gtk.CellRendererText()
+			svg_cols[i].pack_start(cell, gtk.TRUE)
+			svg_cols[i].add_attribute(cell, 'text', i)
 
 		isv = gtk.ScrolledWindow()
 		isv.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
@@ -185,7 +186,7 @@ class SVGManager(gtk.Window):
 		for item in [
 			[gtk.STOCK_ADD,     unicode(_('Add'),    'utf-8'), self.add_ghost,    None],
 			[gtk.STOCK_YES,     unicode(_('FMO'),    'utf-8'), self.check_fmo,    None],
-			[gtk.STOCK_NO,      unicode(_('Use'),    'utf-8'), self.check_fmo,    None],
+			[gtk.STOCK_NO,      unicode(_('Use'),    'utf-8'), self.check_use,    None],
 			[gtk.STOCK_REMOVE,  unicode(_('Remove'), 'utf-8'), self.delete_ghost, [ord('D'), gtk.gdk.CONTROL_MASK]],
 			[gtk.STOCK_REFRESH, unicode(_('Reload'), 'utf-8'), self.reload,       [ord('R'), gtk.gdk.CONTROL_MASK]],
 			[gtk.STOCK_SAVE,    unicode(_('Save'),   'utf-8'), self.save,         [ord('S'), gtk.gdk.CONTROL_MASK]],
@@ -210,21 +211,25 @@ class SVGManager(gtk.Window):
 		self.set_default_size(450, 450)
 		self.add_accel_group(accel_group)
 		self.add(main_hbox)
+		self.clear_info()
 
 	def open(self, widget=None, data=None):
 		self.show_all()
-		self.gn.grab_focus()
+		#self.ghost_list.grab_focus()
 
 	def close(self, widget=None, event=None, data=None):
 		self.hide()
 		return gtk.TRUE
 
+	def browser_changed(self, browser):
+		self.browse_command = browser
+
 	def sort(self, treeviewcol, col_num):
-		id = self.gn_str.get_sort_column_id()
+		id = self.ghost_list_str.get_sort_column_id()
 		if id[0] == col_num:
-			self.gn_str.set_sort_column_id(col_num, not id[1])
+			self.ghost_list_str.set_sort_column_id(col_num, not id[1])
 		else:
-			self.gn_str.set_sort_column_id(col_num, gtk.SORT_DESCENDING)
+			self.ghost_list_str.set_sort_column_id(col_num, gtk.SORT_DESCENDING)
 
 	def search(self, widget=None, data=None):
 		pat = re.compile(self.search_entry.get_text(), re.IGNORECASE)
@@ -232,23 +237,23 @@ class SVGManager(gtk.Window):
 		rng = []
 
 		if self.s_arrow.get_property('arrow-type') == gtk.ARROW_UP:
-			if current == None: current = len(self.gn_str)
+			if current == None: current = len(self.ghost_list_str)
 			rng = range(0, current)
 			rng.reverse()
 		elif self.s_arrow.get_property('arrow-type') == gtk.ARROW_DOWN:
 			if current == None: current = 0
-			rng = range(current+1, len(self.gn_str))
+			rng = range(current+1, len(self.ghost_list_str))
 
 		for i in rng:
 			for j in range(1, 3):
-				m = pat.search(self.gn_str.get_value(self.gn_str.get_iter(i), j))
+				m = pat.search(self.ghost_list_str.get_value(self.ghost_list_str.get_iter(i), j))
 				if m:
-					self.gn.set_cursor(i)
+					self.ghost_list.set_cursor(i)
 					return None
 
 	def is_selected(self):
 		try:
-			treeselection = self.gn.get_selection()
+			treeselection = self.ghost_list.get_selection()
 			[liststore, treeiter] = treeselection.get_selected()
 			if not treeiter:
 				return False
@@ -258,7 +263,7 @@ class SVGManager(gtk.Window):
 
 	def selection(self):
 		if self.is_selected():
-			treeselection = self.gn.get_selection()
+			treeselection = self.ghost_list.get_selection()
 			[liststore, treeiter] = treeselection.get_selected()
 			pathes = liststore.get_path(treeiter)
 			return pathes[0]
@@ -288,18 +293,74 @@ class SVGManager(gtk.Window):
 		return gtk.FALSE
 
 ## 'cursor-changed' signal gtk.TreeView
-	def cc(self, treeview):
+	def cursor_changed(self, treeview):
 		treeview.emit("select-cursor-row", gtk.FALSE)
 
-	EXPAND_QUOTE_REGEX = re.compile('"(.+?)"')
+	def set_info(self, ghost_name='', sakura='', unyu='', craftman='', hpname='', hpurl='', supported='',
+				 action='', text_path='', surface_path='', version='', info=''):
+		status = [
+			['sakura',		sakura],
+			['unyu',		unyu],
+			['craftman',	craftman],
+			['HPname',		hpname],
+			['URL',			hpurl],
+			['supported',	supported],
+			['action',		action],
+			['DEF.',		text_path],
+			['SUF.',		surface_path],
+			['Ver.',		version],
+			['info',		info],
+			]
+
+		self.info_svg_str.clear()
+		for l in status:
+			self.info_svg_str.append(l)
+		self.ghost_name.set_text(ghost_name)
+
+	def clear_info(self):
+		self.sur_button.set_sensitive(gtk.FALSE)
+		self.url_button.set_sensitive(gtk.FALSE)
+		self.sakura_box.set_from_pixbuf(gtk.gdk.pixbuf_new_from_xpm_data(XPM_VANISH))
+		self.unyuu_box.set_from_pixbuf(gtk.gdk.pixbuf_new_from_xpm_data(XPM_VANISH))
+		self.set_info()
+		self.disconnect_buttons()
+
+	def error_info(self, txt_path):
+		self.sur_button.set_sensitive(gtk.FALSE)
+		self.url_button.set_sensitive(gtk.FALSE)
+		self.sakura_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		self.unyuu_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
+		self.set_info(unicode(_(txt_path), 'utf-8') + unicode(_(' is NOT existing!!'), 'utf-8'))
+		self.disconnect_buttons()
+
+	def disconnect_buttons(self):
+		if self.button_id:
+			self.url_button.disconnect(self.button_id)
+		self.button_id = None
+
+		if self.button_id_s:
+			self.sur_button.disconnect(self.button_id_s)
+		self.button_id_s = None
+
 	def select_row(self, treeview, se=gtk.FALSE):
-		ifg = self.gn_str.get_value(self.gn_str.get_iter(self.gn.get_cursor()[0][0]), 3)
-		try:
-			ifg = self.EXPAND_QUOTE_REGEX.match(ifg).group(1)
-		except AttributeError:
-			pass
-		g = self.gm.nominate_ghost_txt(os.path.join(self.base_dir, ifg), \
-									   os.path.join(self.base_dir, 'Ghost'))
+		if len(self.ghost_list_str) <= 0:
+			self.clear_info()
+			return gtk.TRUE
+
+		txt_path = expand_quote(self.ghost_list_str.get_value(self.ghost_list_str.get_iter(self.ghost_list.get_cursor()[0][0]), 3))
+		txt_path = os.path.join(self.ghostmanager.get_ghost_dir(), os.path.basename(txt_path))
+		g = self.ghostmanager.nominate_ghost_txt(txt_path)
+		if g is None:
+			self.error_info(txt_path)
+			#self.clear_info()
+			#self.set_info()
+			#self.info_svg_str.clear()
+			#self.sur_button.set_sensitive(gtk.FALSE)
+			#self.url_button.set_sensitive(gtk.FALSE)
+			#self.ghost_name.set_text(unicode(_(txt_path), 'utf-8') + unicode(_(' is NOT existing!!'), 'utf-8'))
+			#self.sakura_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
+			#self.unyuu_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
+			return gtk.TRUE
 
 		## get and show sakura, unyuu surfaces
 		sur_sakura = g.get_surface(0)
@@ -308,41 +369,42 @@ class SVGManager(gtk.Window):
 		self.sakura_box.set_from_pixbuf(sur_sakura)
 		self.unyuu_box.set_from_pixbuf(sur_unyuu)
 
-		## get and show ghost information
-		sakura = g.get_sakura()
-		unyuu = g.get_unyu()
-		craftman = g.get_craftman()
-		hpname = g.get_hpname()
 		hpurl = g.get_url()
-		surfacefile = g.get_surfacefile()
-		update = g.get_update()
-		ghostname = g.get_ghostname()
-		supported = g.get_supported()
-		sakura_action = g.get_sakura_action()
-		inform = g.get_inform()
+		#self.ghost_name.set_text(g.get_ghostname())
 
-		self.ghost_name.set_text(ghostname)
+		self.set_info(g.get_ghostname(), g.get_sakura(), g.get_unyu(), g.get_craftman(),
+					  g.get_hpname(), hpurl, g.get_supported(), g.get_sakura_action(), txt_path,
+					  os.path.join(self.ghostmanager.get_ghost_dir(), g.get_surfacefile()),
+					  g.get_update(), g.get_inform())
 
-		information = sakura + '\n  &\n' + unyuu + '\n'
-		if craftman:      information = information + '\nby:\t' + craftman
-		if hpname:        information = information + '\nHP Name:\t' + hpname
-		if hpurl:         information = information + '\nHP URL:\n' + hpurl + '\n'
-		if supported:     information = information + '\n' + supported
-		if sakura_action: information = information + '\nSakuraAct:\t' + sakura_action
-		information = information + '\nDEF.:./' + ifg
-		if surfacefile:   information = information + '\nSUF.:./Ghost/' + surfacefile
-		if update:        information = information + '\n\nVer.' + update
-		if inform:        information = information + '\n' + inform
+# 		status = [
+# 			['sakura',		g.get_sakura()],
+# 			['unyu',		g.get_unyu()],
+# 			['craftman',	g.get_craftman()],
+# 			['HPname',		g.get_hpname()],
+# 			['URL',			hpurl],
+# 			['supported',	g.get_supported()],
+# 			['action',		g.get_sakura_action()],
+# 			['DEF.',		txt_path],
+# 			['SUF.',		os.path.join(self.ghostmanager.get_ghost_dir(), g.get_surfacefile())],
+# 			['Ver.',		g.get_update()],
+# 			['info',		g.get_inform()]
+# 			]
 
-		self.info_buffer.set_text(information)
+# 		self.info_svg_str.clear()
+		
+# 		for i in range(len(status)):
+# 			self.info_svg_str.append(status[i])
 
-		if self.button_id:
-			self.url_button.disconnect(self.button_id)
-		self.button_id = None
+# 		if self.button_id:
+# 			self.url_button.disconnect(self.button_id)
+# 		self.button_id = None
 
-		if self.button_id_s:
-			self.sur_button.disconnect(self.button_id_s)
-		self.button_id_s = None
+# 		if self.button_id_s:
+# 			self.sur_button.disconnect(self.button_id_s)
+# 		self.button_id_s = None
+
+		self.disconnect_buttons()
 
 		if hpurl:
 			self.url_button.set_sensitive(gtk.TRUE)
@@ -353,97 +415,153 @@ class SVGManager(gtk.Window):
 		self.sur_button.set_sensitive(gtk.TRUE)
 		self.button_id_s = self.sur_button.connect("clicked", self.show_surface, g)
 
+	def _copy_svg(self, src_file, dest_file):
+		if os.path.exists(src_file) and os.path.exists(dest_file) and \
+			   os.path.samefile(src_file, dest_file):
+			return True
+
+		if os.path.exists(dest_file):
+			d = get_simple_yes_or_no_dialog(unicode(_('Really?'), 'utf-8'),
+											unicode(_('File already exists.'), 'utf-8') + \
+											' ' + unicode(_('overwrite?'), 'utf-8') + \
+											'\n%s' % dest_file, self)
+			d.show()
+			res = d.run()
+			d.destroy()
+			if not res or res != gtk.RESPONSE_YES:
+				return False
+		try:
+			shutil.copyfile(src_file, dest_file)
+		except:
+			return False
+		return True
+
+	def copy_svg(self, src_textfile):
+		src_textfile = os.path.abspath(src_textfile)
+		ghost_dir    = self.ghostmanager.get_ghost_dir()
+
+		if not os.path.exists(src_textfile):
+			open_error_dialog(unicode(_('File not found'), 'utf-8') + \
+							  ': %s' % src_textfile, self)
+			return None
+		src_dir, textfile = os.path.split(src_textfile)
+		try:
+			ghost = self.ghostmanager.nominate_ghost_txt(textfile, src_dir)
+		except:
+			open_error_dialog(unicode(_('Invalid file'), 'utf-8') + \
+							  ': %s' % src_textfile, self)
+			return None
+		imagefile     = ghost.get_surfacefile()
+		src_imagefile = os.path.join(src_dir, imagefile)
+		if not os.path.exists(src_imagefile):
+			open_error_dialog(unicode(_('File not found'), 'utf-8') + \
+							  ': %s' % src_imagefile, self)
+			return None
+
+		dest_dir       = self.ghostmanager.get_ghost_dir()
+		dest_textfile  = os.path.join(ghost_dir, textfile)
+		dest_imagefile = os.path.join(ghost_dir, imagefile)
+
+		if self._copy_svg(src_textfile, dest_textfile):
+			if self._copy_svg(src_imagefile, dest_imagefile):
+				return ghost
+			return None
+		return None
+
 ## if pressed button
 	def add_ghost(self, button):
-		def file_ok_sel(w):
-			filename = filew.get_filename()
-			filew.destroy()
-			## call Ghost(filename)
-			new_svg_dir = os.path.split(filename)[0]
-			g = self.gm.nominate_ghost_txt(filename, new_svg_dir)
-			new_ghost_name = g.get_sakura2()
-
-			if not new_ghost_name:
-				new_ghost_name = g.get_sakura()
-			if not new_ghost_name and not g.get_ghostname():
-				open_error_dialog(unicode(_("Can't open file"), 'utf-8'), self)
-				return -1
-
-			self.gn_str.append([gtk.TRUE, '%s' % new_ghost_name, '',
-					'"%s"' % os.path.join('Ghost', os.path.split(filename)[-1])])
-
-			## added new_svg_txt if not same ghost_dir
-			if not ghost_dir == new_svg_dir:
-				shutil.copyfile(filename, os.path.join(ghost_dir, os.path.split(filename)[-1]))
-				if os.path.exists(os.path.join(new_svg_dir, g.get_surfacefile())):
-					shutil.copyfile(os.path.join(new_svg_dir, g.get_surfacefile()), \
-					         os.path.join(ghost_dir, g.get_surfacefile()))
-			self.gn.set_cursor(len(self.gn_str)-1)
+		ghost_dir = self.ghostmanager.get_ghost_dir()
 
 		filew = gtk.FileSelection()
 		filew.set_select_multiple(gtk.FALSE)
 		filew.connect("destroy", lambda x: filew.destroy())
-
-		ghost_dir = os.path.join(self.base_dir, 'Ghost')
-		if not os.path.exists(ghost_dir):
-			os.makedirs(ghost_dir)
 		filew.set_filename(os.path.normpath(os.path.join(ghost_dir, '*.txt')))
-		filew.ok_button.connect("clicked", file_ok_sel)
-		filew.cancel_button.connect("clicked", lambda x: filew.destroy())
 		filew.show()
+		res = filew.run()
+		src_svg_file = filew.get_filename()
+		filew.destroy()
+
+		if res != gtk.RESPONSE_OK:
+			return gtk.TRUE
+
+		ghost = self.copy_svg(src_svg_file)
+		if not ghost:
+			sys.stderr.write("!!! couldn't copy %s\n" % src_svg_file)
+			return gtk.TRUE
+
+		ghost_name = ghost.get_sakura2()
+		if not ghost_name:
+			ghost_name = ghost.get_sakura()
+
+		if not ghost_name and not ghost.get_ghostname():
+			open_error_dialog(unicode(_("Couldn't open file"), 'utf-8'), self)
+			return gtk.TRUE
+
+		self.ghost_list_str.append([gtk.TRUE, ghost_name, '',
+							'"%s"' % os.path.join(os.path.basename(ghost_dir),
+												  os.path.basename(src_svg_file))])
+		self.ghost_list.set_cursor(len(self.ghost_list_str)-1)
+		return gtk.TRUE
 
 	def check_fmo(self, button):
 		try:
-			i = self.gn.get_cursor()[0][0]
-			now = self.gn.get_selection()
+			i = self.ghost_list.get_cursor()[0][0]
+			now = self.ghost_list.get_selection()
 			[model, iter] = now.get_selected()
 			if model.get_value(iter, 2) == 'FMO':
 				self.delete_fmo(model, iter)
 			else:
 				self.set_fmo(model, iter)
-			self.gn.set_cursor(i)
-			self.gn.grab_focus()
+			self.ghost_list.set_cursor(i)
+			#self.ghost_list.grab_focus()
 		except:
 			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)
 
 	def check_use(self, button):
 		try:
-			i = self.gn.get_cursor()[0][0]
-			now = self.gn.get_selection()
+			i = self.ghost_list.get_cursor()[0][0]
+			now = self.ghost_list.get_selection()
 			[model, iter] = now.get_selected()
 			if model.get_value(iter, 0):
 				self.unuse_ghost(model, iter)
 			else:
 				self.use_ghost(model, iter)
-			self.gn.set_cursor(i)
-			self.gn.grab_focus()
+			self.ghost_list.set_cursor(i)
+			#self.ghost_list.grab_focus()
 		except:
 			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)
 
 	def delete_ghost(self, button):
 		try:
-			i = self.gn.get_cursor()[0][0]
-			self.gn_str.remove(self.gn_str.get_iter(i))
-			if i == len(self.gn_str):
-				self.gn.set_cursor(i-1)
+			i = self.ghost_list.get_cursor()[0][0]
+			self.ghost_list_str.remove(self.ghost_list_str.get_iter(i))
+			if i == len(self.ghost_list_str):
+				self.ghost_list.set_cursor(i-1)
 			else:
-				self.gn.set_cursor(i)
+				self.ghost_list.set_cursor(i)
 		except:
 			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)
 
 	def reload(self, button):
 		self.get_config()
-		self.gn.grab_focus()
+		self.app.notify_svg_changed()
+		#self.ghost_list.grab_focus()
 
 	def save(self, button):
 		self.set_config()
 		self.app.notify_svg_changed()
 
+	def get_ghost(self, ifghost):
+		return self.ghostmanager.nominate_ghost(ifghost)
+
+	def get_fmo_ghostnames(self):
+		return self.ghostmanager.get_fmo_ghostnames()
+
 	def check_use_toggle(self, cellrenderer, path):
-		model = self.gn_str
+		model = self.ghost_list_str
 		iter = model.get_iter(path)
 		try:
-			if self.gn_str.get_value(iter, 0):
+			if self.ghost_list_str.get_value(iter, 0):
 				self.unuse_ghost(model, iter)
 			else:
 				self.use_ghost(model, iter)
@@ -452,93 +570,43 @@ class SVGManager(gtk.Window):
 
 	def url_jump(self, button, hpurl):
 		try:
-			os.system(self.prefs.get('browser') % hpurl + ' &')
+			os.system(self.browse_command % hpurl + ' &')
 		except:
 			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)
 
 	def show_surface(self, button, ghost):
-		path = os.path.join(self.base_dir, 'Ghost', ghost.get_surfacefile())
+		path = os.path.join(self.ghostmanager.get_ghost_dir(), ghost.get_surfacefile())
 		try:
 			pixbuf = gtk.gdk.pixbuf_new_from_file(path)
 		except:
 			return
 		ss = ShowingSurface(self.app, pixbuf, ghost)
 
-## operate the CONFIG_FILE
-	def get_config(self, config_file='ghost.txt'):
-		self.gn_str.clear()
-		## read a config_file, make a list, and return the list
-		config_file = os.path.join(self.base_dir, config_file)
-		try:
-			file = open(config_file)
-		except IOError:
-			print "cannot open ", config_file
-			return None
+	def get_config(self):
+		self.ghost_list_str.clear()
+		self.ghostmanager.generate_dictionary()
+		for line in self.ghostmanager.get_dict().values():
+			self.ghost_list_str.append(line)
 
-		## read ghost.txt
-		flag = False
-		while 1:
-			#read file and hogehoge
-			line = file.readline()
-			if not line:
-				continue
-			if line[:4] == '/EOF':
-				break
-			if line[:7] == 'INSTALL' or not flag:
-				flag = True
-				continue
-
-			key = line.strip()
-			if not key:
-				continue
-			key = unicode(key, 'sjis', 'replace')
-			key = key.replace('\\', r'/')
-			tmp = key.split(',')
-			if tmp[0] == 'GHOST':
-				self.gn_str.append([gtk.TRUE, tmp[1], tmp[2], tmp[3]])
-			else:
-				self.gn_str.append([gtk.FALSE, tmp[1], tmp[2], tmp[3]])
-		file.close()
-
-	def set_config(self, config_file='ghost.txt'):
-		## save config, (write to CONFIG_FILE)
-		config_file = os.path.join(self.base_dir, config_file)
-		try:
-			file = open(config_file, "w")
-		except IOError:
-			print "can't write to file"
-
-		file.write('INSTALL,,,"Ghost\\"\n')
-		for i in range(0, len(self.gn_str)):
-			iter = self.gn_str.get_iter(i)
-			def_file = self.gn_str.get_value(iter, 3)
-			def_file = def_file.replace(r'/', '\\')
-			if self.gn_str.get_value(iter, 0):
-				ghost = 'GHOST'
-			else:
-				ghost = '#GHOST'
-			tmp = ghost + ',' + self.gn_str.get_value(iter, 1) + ',' + self.gn_str.get_value(iter, 2) + ',' + def_file + '\n'
-			tmp = tmp.encode('sjis')
-			file.write(tmp)
-		file.write('/EOF')
-		file.close()
+	def set_config(self):
+		self.ghostmanager.save_dictionary(self.ghost_list_str)
 
 	def use_ghost(self, model, iter):
 		## delete '#' from  first item '#GHOST'
-		self.gn_str.set_value(iter, 0, gtk.TRUE)
+		self.ghost_list_str.set_value(iter, 0, gtk.TRUE)
 
 	def unuse_ghost(self, model, iter):
 		## insert '#' in first item 'GHOST'
-		self.gn_str.set_value(iter, 0, gtk.FALSE)
+		self.ghost_list_str.set_value(iter, 0, gtk.FALSE)
 
 	def set_fmo(self, model, iter):
 		## insert 'FMO' the third item
-		self.gn_str.set_value(iter, 2, 'FMO')
+		self.ghost_list_str.set_value(iter, 2, 'FMO')
 
 	def delete_fmo(self, model, iter):
 		## delete 'FMO' the third item 'FMO'
-		self.gn_str.set_value(iter, 2, '')
+		self.ghost_list_str.set_value(iter, 2, '')
 
 if __name__ == "__main__":
-	sm = SVGManager()
+	sm = SVGManagerWindow()
 	gtk.main()
