@@ -12,7 +12,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: ghost.py,v 1.12 2004/10/03 04:14:39 atzm Exp $
+# $Id: ghost.py,v 1.13 2004/11/13 18:05:01 tkawa Exp $
 #
 
 import gtk
@@ -31,6 +31,7 @@ class Ghost:
 		self.svg_dir     = svg_dir
 		self.surface_dic = {}
 		self.reverse_dic = {}
+		self.sequential_dic = {}
 
 		self.sakura      = None
 		self.unyu        = None
@@ -201,7 +202,21 @@ class Ghost:
 
 							last_splite = s1 + 1
 
+				# get a sequential images
+				elif line[:11] == 'sequential=':
+					### NOTICE!!
+					### self.sequential_dic type of STRING
+					# delete "\[" and "\]"
+					seque = re.match('\[(.*)\]', line[11:]).group(1)
+					sequential_index, sequential_group = re.split('=', seque, 1)
+					# we need make and set dictionary that is {'4':'100>>101>>102>>*'} for example
+					#print 'hoge', sequential_index, sequential_group
+					self.sequential_dic[int(sequential_index)] = sequential_group
+
+					self.debugprint('sequential splite', self.surfacefile)
+
 		#print self.surface_dic
+		#print self.sequential_dic
 
 		image_path = os.path.join(self.svg_dir, self.surfacefile)
 		try:
@@ -317,3 +332,40 @@ class Ghost:
 		for i in self.surface_dic.keys():
 			if not self.reverse_dic.has_key(i):
 				self.reverse_dic[self.surface_dic[i]] = i
+
+	def check_ss(self, surface_id):
+		if not self.sequential_dic or not surface_id:
+			return False
+		try:
+			id = int(surface_id)
+			splite_id = self.surface_dic[id]
+		except KeyError:
+			return False
+		return self.sequential_dic.has_key(splite_id)
+
+	def get_surface_s(self, splite_id, side=SIDE_SAKURA):	## for Sequential Splite
+		id = int(splite_id)
+		if self.ghost_pixbuf is None:
+			return None
+		else:
+			if id == -1:
+				area = gtk.gdk.pixbuf_new_from_xpm_data(XPM_VANISH)
+			else:
+				svg_width  = self.ghost_pixbuf.get_width()
+				svg_height = self.ghost_pixbuf.get_height()
+
+				x = svg_width / SIZE_SURFACE_H
+				y = svg_height / SIZE_SURFACE_V
+				a = id % x
+				b = id / x
+
+				area = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, gtk.TRUE, 8, 48, 64)
+				self.ghost_pixbuf.copy_area(SIZE_SURFACE_H * a, SIZE_SURFACE_V * b,
+											SIZE_SURFACE_H, SIZE_SURFACE_V, area, 0, 0)
+		return area
+
+	def get_ss_group(self, surface_id):
+		id = int(surface_id)
+		sequential_def = self.sequential_dic[self.surface_dic[id]]
+		splite = re.split('[>]', sequential_def)
+		return splite
