@@ -10,9 +10,11 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: viewercommon.py,v 1.2 2004/06/23 12:00:14 atzm Exp $
+# $Id: viewercommon.py,v 1.3 2004/10/03 04:14:39 atzm Exp $
 #
 
+import re
+
 # debug
 DEBUG = False
 
@@ -44,3 +46,14 @@ EVENT_CLEAR = 4
 NAME_SAKURA          = unicode('さくら', 'utf-8')
 NAME_SAKURA_KUROI    = unicode('黒衣さくら', 'utf-8')
 NAME_SAKURA_KAWAKAMI = unicode('川上さくら', 'utf-8')
+NAME_SAKURA_DOT      = unicode('．さくら', 'utf-8')
+NAME_SAKURA_DOT_     = unicode('旧・．さくら', 'utf-8')
+
+EXPAND_QUOTE_REGEX = re.compile('"(.*?)"')
+def expand_quote(text):
+	try:
+		text = EXPAND_QUOTE_REGEX.match(text).group(1)
+	except AttributeError:
+		pass
+	return text	
+
